package cn.com.qmth.print.manage.utils;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;

/**
 * 路径工具
 *
 * @author 
 */
public class PathUtil {

	private final static Logger log = LoggerFactory.getLogger(PathUtil.class);

	/**
	 * 获取标准路径
	 *
	 * @author 
	 * @param path
	 * @return
	 */
	public static String getCanonicalPath(String path) {
		path = path.replaceAll("\\\\+", "/");
		path = path.replaceAll("/+", "/");
		return path;
	}

	/**
	 * 以"/"开头
	 *
	 * @author 
	 * @param path
	 * @return
	 */
	public static String startsWithSeparator(String path) {
		path = getCanonicalPath(path);
		if (path.startsWith("/")) {
			return path;
		}
		return "/" + path;
	}

	/**
	 * 不以"/"开头
	 *
	 * @author 
	 * @param path
	 * @return
	 */
	public static String startsWithoutSeparator(String path) {
		path = getCanonicalPath(path);
		if (path.startsWith("/")) {
			return path.substring(1);
		}
		return path;
	}

	/**
	 * 以"/"结尾
	 *
	 * @author 
	 * @param path
	 * @return
	 */
	public static String endsWithSeparator(String path) {
		path = getCanonicalPath(path);
		if (path.endsWith("/")) {
			return path;
		}
		return path + "/";
	}

	/**
	 * 不以"/"结尾
	 *
	 * @author 
	 * @param path
	 * @return
	 */
	public static String endsWithoutSeparator(String path) {
		path = getCanonicalPath(path);
		if (path.endsWith("/")) {
			return path.substring(0, path.length() - 1);
		}
		return path;
	}

	/**
	 * 获取路径
	 *
	 * @author 
	 * @param file
	 * @return
	 */
	public static String getCanonicalPath(File file) {
		try {
			return file.getCanonicalPath();
		} catch (IOException e) {
			throw new RuntimeException("Fail to get canonical path.", e);
		}

	}

	/**
	 * 获取当前路径
	 *
	 * @author 
	 * @return
	 * @throws IOException
	 */
	public static String currentPath() {
		File directory = new File(". ");
		return getCanonicalPath(directory);
	}

	/**
	 * 获取资源路径
	 *
	 * @author 
	 * @param resourceName
	 * @return
	 */
	public static String getResoucePath(String resourceName) {
		try {
			ClassLoader classLoader = PathUtil.class.getClassLoader();

			URL url = classLoader.getResource(resourceName);
			if (null != url) {
				String path = URLDecoder.decode(url.getPath(), "UTF-8");
				return path;
			} else {
				return null;
			}
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static InputStream getStream(String path) {
		try {
			ClassLoader classLoader = PathUtil.class.getClassLoader();
			URL url = classLoader.getResource(path);
			return url.openStream();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public static void outputFile(HttpServletResponse response, InputStream inputStream, String fileName) {
		try {
			BufferedInputStream br = new BufferedInputStream(inputStream);
			String fName = URLEncoder.encode(fileName, "UTF-8");

			response.reset();
			response.setContentType("application/x-msdownload");
			response.setHeader("Content-Disposition", "attachment; filename=" + fName);
			IOUtils.copy(br, response.getOutputStream());
			br.close();
//            outStream.close();
		} catch (IOException e) {
			log.error("请求出错:{}", e);
		}
	}

	/**
	 * 获取windows盘符
	 *
	 * @author 
	 * @param path
	 * @return
	 */
	public static String getDrive(String path) {
		if (path.matches("[a-zA-Z]:[\\\\/].*")) {
			return path.substring(0, 2);
		} else {
			throw new RuntimeException("Path is not a windows path.");
		}
	}

	/**
	 * 拼接路径
	 *
	 * @author 
	 * @param fragments
	 * @return
	 */
	public static String joinUrl(String... fragments) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < fragments.length; i++) {
			String cur = fragments[i].trim();
			if (cur.endsWith("/")) {
				cur = cur.substring(0, cur.length() - 1);
			}
			if (0 == i) {
				sb.append(cur);
			} else if (cur.startsWith("/")) {
				sb.append(cur);
			} else {
				sb.append("/").append(cur);
			}
		}

		return sb.toString();
	}

}
